const updateDeviceIds = [
  {
    vendorId: 0xface,
    productId: 0x2109,
    usage: 0x01,
    usagePage: 0xff00
  },
  {
    vendorId: 0x1abe,
    usage: 0x01,
    usagePage: 0x0002
  }
  /*
  {
    vendorId: 0x054c,
    productId: 0x0ce6
  }
  {
    vendorId: 0x054c,
    productId: 0x0ba0
  },
  {
    vendorId: 0x054c,
    productId: 0x05c4
  },
  {
    vendorId: 0x054c,
    productId: 0x09cc
  },
  {
    vendorId: 0x054c,
    productId: 0x05c5
  },
  {
    vendorId: 0x054c,
    productId: 0x0ba0
  }
  */
];

let hidDevice = null;
let selectFile = null;
//let cmdIdx = 0;
let speedTestCnt = 0;
let downloadCnt = -1;
let downloadSum = 0;
let configupdateWrite = 0;
let configupdateCnt = -1;
let configupdateSum = 0;

let buttonConnectDevice = document.getElementById("connect-device");
let inputVersion = document.getElementById("versionShow");
//let buttonSelectFile = document.getElementById("select-file");
let buttonDownload = document.getElementById("download");
let processDownload = document.getElementById("download-process");
let buttonSelectFile = document.getElementById("select-file");
//let buttonSpeedTest = document.getElementById("speed-test");
//let processSpeedTest = document.getElementById("speed-test-process");
let buttonConfigupdate = document.getElementById("config-update");
let processConfigprocess = document.getElementById("config-process");

(async () => {
  setupButton();
})();

function setupButton() {
  buttonConnectDevice.addEventListener("click", async () => {
    let devices;
    try {
      devices = await navigator.hid.requestDevice({
        filters: updateDeviceIds
      });
      initDevice(devices[0]);
    } catch (e) {
      console.log("buttonConnectDevice onClick error" + e);
    }
  });

  buttonDownload.disabled = true;
  buttonDownload.addEventListener("click", async () => {
    try {
      if (downloadCnt < 0) {
        downloadCnt = ((selectFile.byteLength + 511) & 0xfffffe00) / 512;
        downloadSum = downloadCnt;
        doDownload();
      }
      console.log("buttonDownload onClick ...");
    } catch (e) {
      console.log("buttonDownload onClick error" + e);
    }
  });

  buttonSelectFile.addEventListener("change", async () => {
    selectFile = null;
    if (buttonSelectFile.files.length > 0) {
      let file = buttonSelectFile.files[0];
      let reader = new FileReader();
      reader.onload = function () {
        initFile(reader.result);
      };
      reader.readAsArrayBuffer(file);
    } else {
      initFile(null);
    }
  });

  /*
  //buttonSpeedTest.disabled = true;
  buttonSpeedTest.addEventListener("click", async () => {
    try {
      if (hidDevice != null && !speedTestCnt) {
        speedTestCnt = 1024;
        processSpeedTest.value = 0;
        console.log("Speed Test " + speedTestCnt);
        sendCmdPing();
      }
      console.log("buttonSpeedTest onClick ...");
    } catch (e) {
      console.log("buttonSpeedTest onClick error" + e);
    }
  });
  */

  buttonConfigupdate.addEventListener("click", async () => {
    try {
      if (configupdateCnt < 0) {
        configupdateWrite = 1;
        configupdateCnt = 5;
        configupdateSum = configupdateCnt;
        doConfigupdate();
      }
      console.log("buttonConfigupdate onClick ...");
    } catch (e) {
      console.log("buttonConfigupdate onClick error" + e);
    }
  });
}

async function initDevice(device) {
  hidDevice = device;

  if (hidDevice) {
    speedTestCnt = 0;
    downloadCnt = -1;
    downloadSum = 0;
    configupdateWrite = 0;
    configupdateCnt = -1;
    configupdateSum = 0;

    if (!hidDevice.opened) {
      hidDevice.open();
    }
    hidDevice.oninputreport = (event) => inputReportHandler(event);
  }

  console.log(hidDevice);

  /*
  if (hidDevice != null) {
    buttonSpeedTest.disabled = false;
  } else {
    buttonSpeedTest.disabled = true;
  }
  */

  if (hidDevice != null && selectFile != null) {
    buttonDownload.disabled = false;
  } else {
    buttonDownload.disabled = true;
  }

  setTimeout(function () {
    updateConfig();
  }, 100);
}

async function inputReportHandler(event) {
  const { data, device, reportId } = event;
  parseReportHandler(data, device, reportId);

  /*
  if (speedTestCnt) {
    speedTestCnt = speedTestCnt - 1;
    if (speedTestCnt > 0) {
      console.log("Speed Test " + speedTestCnt);
      sendCmdPing();
    }
    processSpeedTest.value =
      processSpeedTest.max - (speedTestCnt * processSpeedTest.max) / 1024;
  } else */
  if (downloadCnt >= 0) {
    doDownload();
  }
  if (configupdateCnt >= 0) {
    doConfigupdate();
  }
}

async function initFile(file) {
  if (file != null) {
    selectFile = file;
    console.log(selectFile);

    if (hidDevice != null && selectFile != null) {
      buttonDownload.disabled = false;
    }
  } else {
    buttonDownload.disabled = true;
  }
}

async function doDownload() {
  if (downloadCnt === 0) {
    downloadCnt = -1;
    console.log("DownLoad Finish");

    configupdateWrite = 0;
    configupdateCnt = 6;
    configupdateSum = configupdateCnt;
    return;
  }

  //console.log("DownLoad " + (downloadSum - downloadCnt) + "/" + downloadSum);

  var start = 512 * (downloadSum - downloadCnt);
  var end = Math.min(start + 512, selectFile.byteLength + 1);
  var block = new Uint8Array(selectFile.slice(start, end));
  sendCmdDownload512(block, start);

  downloadCnt--;

  processDownload.value =
    processDownload.max - (downloadCnt * processDownload.max) / downloadSum;
}

async function doConfigupdate() {
  if (configupdateWrite) {
    console.log("doConfigupdate write, " + configupdateCnt);
    if (configupdateCnt === 5) {
      sendCmdSetConfigStr(
        updateDeviceIds[0].productId,
        "wifi.ssid",
        inputWifissid.value
      );
    } else if (configupdateCnt === 4) {
      sendCmdSetConfigStr(
        updateDeviceIds[0].productId,
        "wifi.passwd",
        inputWifipasswd.value
      );
    } else if (configupdateCnt === 3) {
      if (inputRppaired.checked) {
        sendCmdSetConfigStr(updateDeviceIds[0].productId, "rp.paired", "1");
      } else {
        sendCmdSetConfigStr(updateDeviceIds[0].productId, "rp.paired", "");
      }
    } else if (configupdateCnt === 2) {
      sendCmdSetConfigStr(
        updateDeviceIds[0].productId,
        "rp.psnid",
        inputRppsnid.value
      );
    } else if (configupdateCnt === 1) {
      sendCmdSetConfigStr(
        updateDeviceIds[0].productId,
        "rp.pin",
        inputRppin.value
      );
    } else if (configupdateCnt === 0) {
      configupdateWrite = 0;
      configupdateCnt = 5;
      configupdateSum = configupdateCnt;
    }
  }
  if (configupdateWrite === 0) {
    console.log("doConfigupdate read, " + configupdateCnt);
    if (configupdateCnt === 6) {
      sendCmdGetConfig(updateDeviceIds[0].productId, "version");
    } else if (configupdateCnt === 5) {
      sendCmdGetConfig(updateDeviceIds[0].productId, "wifi.ssid");
    } else if (configupdateCnt === 4) {
      sendCmdGetConfig(updateDeviceIds[0].productId, "wifi.passwd");
    } else if (configupdateCnt === 3) {
      sendCmdGetConfig(updateDeviceIds[0].productId, "rp.paired");
    } else if (configupdateCnt === 2) {
      sendCmdGetConfig(updateDeviceIds[0].productId, "rp.psnid");
    } else if (configupdateCnt === 1) {
      sendCmdGetConfig(updateDeviceIds[0].productId, "rp.pin");
    }
  }

  configupdateCnt--;

  if (configupdateCnt >= 0) {
    processConfigprocess.value =
      processConfigprocess.max -
      (configupdateCnt * processConfigprocess.max) / configupdateSum;
  }
}

async function sendCmdPing() {
  const bootloader_cmd_ping = [0x14, 0x12, 0x01, 0x00, 0x00];
  hidDevice.sendReport(0, new Uint8Array(bootloader_cmd_ping));
}

async function sendCmdDownload512(block, pos) {
  const bootloader_cmd_download512_head = [0x00, 0x00, 0x02, 0x01, 0x06, 0x02];
  var arrayHead = new Uint8Array(bootloader_cmd_download512_head);
  var arraySize = new Uint8Array(2);
  arraySize[0] = 0x00;
  arraySize[1] = 0x02;
  var arrayPos = new Uint8Array(4);
  arrayPos[0] = pos & 0xff;
  arrayPos[1] = (pos & 0xff00) >> 8;
  arrayPos[2] = (pos & 0xff0000) >> 16;
  arrayPos[3] = (pos & 0xff000000) >> 32;
  var arrayCmd = new Uint8Array(
    arrayHead.byteLength + arraySize.byteLength + arrayPos.byteLength + 512
  );
  arrayCmd.set(arrayHead);
  arrayCmd.set(arraySize, arrayHead.byteLength);

  arrayCmd.set(arrayPos, arrayHead.byteLength + arraySize.byteLength);
  arrayCmd.set(
    block,
    arrayHead.byteLength + arraySize.byteLength + arrayPos.byteLength
  );
  //console.log(arrayCmd);
  hidDevice.sendReport(0, arrayCmd);
}

let inputWifissid = document.getElementById("wifi-ssid");
let inputWifipasswd = document.getElementById("wifi-passwd");
let inputRppsnid = document.getElementById("rp-psnid");
let inputRppin = document.getElementById("rp-pin");
let inputRppaired = document.getElementById("rp-paired");

async function updateConfig() {
  if (hidDevice) {
    configupdateWrite = 0;
    configupdateCnt = 6;
    configupdateSum = configupdateCnt;
    doConfigupdate();

    inputWifissid.disabled = false;
    inputWifipasswd.disabled = false;
    inputRppsnid.disabled = false;
    inputRppin.disabled = false;
    //inputRppaired.disabled = false;
    buttonConfigupdate.disabled = false;
  } else {
    inputWifissid.disabled = true;
    inputWifipasswd.disabled = true;
    inputRppsnid.disabled = true;
    inputRppin.disabled = true;
    inputRppaired.disabled = true;
    buttonConfigupdate.disabled = true;
    processConfigprocess.value = 0;
  }
}

async function parseReportHandler(data, device, reportId) {
  console.log(device);
  console.log(reportId);
  console.log(data);

  //const req_idx = data.getUint16(0, true);
  const req_command = data.getUint16(2, true);
  //const req_data_len = data.getUint16(4, true);

  switch (req_command) {
    case 0x0000: // BOOTLOADER_CMD_NOT_SUPPORT
      break;
    case 0x0001: // BOOTLOADER_CMD_PING
      break;
    case 0x0002: // BOOTLOADER_CMD_FLUSH
      break;
    case 0x0003: // BOOTLOADER_CMD_GET_INFO
      break;
    case 0x0100: // BOOTLOADER_CMD_DOWNLOAD
      break;
    case 0x0101: // BOOTLOADER_CMD_DOWNLOAD_END
      break;
    case 0x0102: // BOOTLOADER_CMD_DOWNLOAD_512
      break;
    case 0x0200: // BOOTLOADER_CMD_UPLOAD
      break;
    case 0x0300: // BOOTLOADER_CMD_SET_CONFIG
      break;
    case 0x0301: // BOOTLOADER_CMD_GET_CONFIG
      console.log(data.getUint16(6, true));
      if (data.getUint16(6, true) === updateDeviceIds[0].productId) {
        const size = data.getUint16(8, true);
        const buffer = new Uint8Array(data.buffer);
        const cmdStr = Uint8ArrayToString(buffer.slice(10, 10 + 16));
        const configStr = Uint8ArrayToString(buffer.slice(26, 26 + size));
        console.log("Get Config: " + cmdStr + ": " + configStr);

        switch (cmdStr) {
          case "wifi.ssid":
            inputWifissid.value = configStr;
            break;
          case "wifi.passwd":
            inputWifipasswd.value = configStr;
            break;
          case "rp.paired":
            console.log(data.getUint8(26));
            if (data.getUint8(26) > 0) {
              inputRppaired.checked = true;
            } else {
              inputRppaired.checked = false;
            }
            break;
          case "rp.psnid":
            inputRppsnid.value = configStr;
            break;
          case "rp.pin":
            inputRppin.value = configStr;
            break;
          case "version":
            inputVersion.value = configStr;
            break;
          default:
            console.log("Get Unknown " + cmdStr);
            break;
        }
      }
      break;
    default:
      break;
  }
}

function Uint8ArrayToString(fileData) {
  var dataString = "";
  for (var i = 0; i < fileData.length; i++) {
    if (fileData[i] === 0) {
      break;
    }
    dataString += String.fromCharCode(fileData[i]);
  }
  return dataString;
}

function stringToUint8Array(str) {
  var arr = [];
  for (var i = 0, j = str.length; i < j; ++i) {
    arr.push(str.charCodeAt(i));
  }
  var tmpUint8Array = new Uint8Array(arr);
  return tmpUint8Array;
}

async function sendCmdSetConfigStr(pid, configStr, dataStr) {
  var arrayConfigStr = stringToUint8Array(configStr);
  var arrayDataStr = stringToUint8Array(dataStr);
  console.log(dataStr);
  console.log(arrayDataStr);

  const bootloader_cmd_setconfig_head = [0x00, 0x00, 0x00, 0x03];
  var arrayHead = new Uint8Array(bootloader_cmd_setconfig_head);
  var arrayDataLen = new Uint8Array(2);
  arrayDataLen[0] = (4 + 16 + arrayDataStr.byteLength) & 0xff;
  arrayDataLen[1] = ((4 + 16 + arrayDataStr.byteLength) & 0xff00) >> 8;
  var arrayPid = new Uint8Array(2);
  arrayPid[0] = pid & 0xff;
  arrayPid[1] = (pid & 0xff00) >> 8;
  var arraySize = new Uint8Array(2);
  arraySize[0] = arrayDataStr.byteLength & 0xff;
  arraySize[1] = (arrayDataStr.byteLength & 0xff00) >> 8;

  var arrayCmd = new Uint8Array(
    arrayHead.byteLength +
      arrayDataLen.byteLength +
      arrayPid.byteLength +
      arraySize.byteLength +
      16 +
      arrayDataStr.byteLength
  );
  arrayCmd.set(arrayHead);
  arrayCmd.set(arrayDataLen, arrayHead.byteLength);
  arrayCmd.set(arrayPid, arrayHead.byteLength + arrayDataLen.byteLength);
  arrayCmd.set(
    arraySize,
    arrayHead.byteLength + arrayDataLen.byteLength + arrayPid.byteLength
  );
  arrayCmd.set(
    arrayConfigStr,
    arrayHead.byteLength +
      arrayDataLen.byteLength +
      arrayPid.byteLength +
      arraySize.byteLength
  );
  arrayCmd.set(
    arrayDataStr,
    arrayHead.byteLength +
      arrayDataLen.byteLength +
      arrayPid.byteLength +
      arraySize.byteLength +
      16
  );

  setTimeout(function () {
    hidDevice.sendReport(0, arrayCmd);
  }, 25);
}

async function sendCmdGetConfig(pid, configStr) {
  var arrayConfigStr = stringToUint8Array(configStr);
  const bootloader_cmd_getconfig_head = [0x00, 0x00, 0x01, 0x03, 0x12, 0x00];
  var arrayHead = new Uint8Array(bootloader_cmd_getconfig_head);
  var arrayPid = new Uint8Array(2);
  arrayPid[0] = pid & 0xff;
  arrayPid[1] = (pid & 0xff00) >> 8;
  var arrayCmd = new Uint8Array(
    arrayHead.byteLength + arrayPid.byteLength + arrayConfigStr.byteLength
  );
  arrayCmd.set(arrayHead);
  arrayCmd.set(arrayPid, arrayHead.byteLength);
  arrayCmd.set(arrayConfigStr, arrayHead.byteLength + arrayPid.byteLength);

  setTimeout(function () {
    hidDevice.sendReport(0, arrayCmd);
  }, 25);
}
